function [MSE,I,Yfpred]=predico(flag,X,Y,Xf,Yf,input1,input2,input3,input4)

%predico: Bayesian Variable Selection
%         Prediction by Least Squares, Bayesian Model Averaging 
%	  or BMA with selection.
%***********************************************************************
%INPUTS:
% flag,		- flag='LS' to perform least squares prediction
%		- flag='BMA' to  perfom Bayesian Model Averaging 
%		- flag='RBMA' to  perfom BMA with selection
% X,Y,		training data
% Xf,Yf,	validation data
% input1,	- vector of 0/1 or cell with vars numbering
%		  (for flag='LS')
%		- matrix or cell of distinct ordered visited models
%		  (for flag='BMA')
%		- output matrix of atteso.m 
%		  (for flag='RBMA')
% input2, 	- vector of posterior probability of visited models
%		  (for flag='BMA')
%		- vector with selected variables (use ricerca.m)
%		  (for flag='RBMA')
% input3,	- scalar or vector of std's or full var-cov matrix
%		  (for flag='BMA')
%		- scalar, indicates which lower percentile of non-zero 
%		  eigenvalues of X should be used
%		  (for flag='RBMA')
% input4,	- scalar, number of best models for model avaraging
%		  (for flag='BMA')
%
%OUTPUTS:
% MSE,		- LS predictions for flag='LS' 
%		- Bayes model averaging predictions for flag='BMA' 
%		- Bayes model averaging predictions with selection flag='RBMA'
% I,		- vars numbering for flag='LS' 
%		- all variables used in prediction for flag='BMA' 
%		- same as input2 for flag='RBMA' 
% Yfpred,	- prediction of Yf
%
%USAGE:
% [MSE,I,Yfpred]=predico('LS',X,Y,Xf,Yf,input1);
% [MSE,I,Yfpred]=predico('BMA',X,Y,Xf,Yf,input1,input2,input3,input4);
% [MSE,I,Yfpred]=predico('RBMA',X,Y,Xf,Yf,input1,input2,input3);
%
%NOTES:
% flag='BMA' works for Hg scalar (g-prior), Hg vector of std's (diagonal 
% prior on regression coeffs) and Hg full matrix (non-diagonal prior)
%
%REFERENCE: see readme_fast
%
%
%Copyright (c) 1997-2003 Marina Vannucci.
%**********************************************************************

if nargin==6
	gammodel=input1;
	switch class(gammodel)
		case 'cell'
			I=str2num(gammodel{1});			
		otherwise
			I=find(gammodel);
	end	
elseif nargin==9
	GammaD=input1;
	PostGamD=input2;
	Hg=input3;
	numero=input4;
elseif nargin==8
	EMg=input1;
	restr=input2;
	pct=input3;
end
	
[nOssP,p]=size(Xf);
[nOss]=size(X,1);
[q]=size(Y,2);

switch flag
	case 'LS'
%-------------------------------------- LS prediction with 'gamma' model
		
		nonnulle=length(I);
		Xfg=Xf(:,I);
		Xg=X(:,I);
		if nonnulle>nOss
			disp(' ')
			disp('Least square prediction with more variables')
			disp('than observations. Use generalized inverse')
			disp(' ')
			Beta=pinv(Xg'*Xg)*Xg'*Y;
			Yfpred=Xfg*Beta;
			residual=Yf-Yfpred;
		else 
			[Qqr Rqr]=qr(Xg);
			Zeta=Qqr(:,1:nonnulle)'*Y;
			U=Qqr(:,1:nonnulle)'*Xg;
			Beta=inv(U)*Zeta;  
			Yfpred=Xfg*Beta;
			residual=Yf-Yfpred;
		end		
		
	case 'BMA'
%-------------------------------------- BMA prediction with 'numero' models
    
		if (size(Hg,1)==1 & size(Hg,2)==1)
			%--------------------------- Hg scalar - gPrior 
			Ytilde=Y;

		elseif (size(Hg,1)==1 | size(Hg,2)==1)
			%--------------------------- Hg vector of std's 
			Ytilde=cat(1,Y, zeros(p,q));			 	
		else				 
			%--------------------------- Hg full matrix 
			Ytilde=cat(1,Y, zeros(p,q));
		end		
		I=[];
		Yfpred=zeros(nOssP,q);
		costante=sum(PostGamD(1:numero));
		for i=1:numero
			switch class(GammaD)
			case 'cell'
				Ii=str2num(GammaD{i});
			otherwise
				Ii=find(GammaD(i,:));
			end
			nonnulle=length(Ii);
			if (size(Hg,1)==1 & size(Hg,2)==1)
			%--------------------------- Hg scalar - gPrior 
				moltiplica=Hg/(Hg+1);
				Xtilde=	X(:,Ii);
						
			elseif (size(Hg,1)==1 | size(Hg,2)==1)	
			%--------------------------- Hg vector of std's 
			
				moltiplica=diag(Hg(Ii));  
				XH0=X(:,Ii)*moltiplica;
				sotto=zeros(p,nonnulle);
				sotto(Ii,:)=eye(nonnulle);    
				Xtilde=cat(1, XH0, sotto);	
			else
			%--------------------------- Hg full matrix 
				moltiplica=sqrtm(Hg(Ii,Ii));
				XH0=X(:,Ii)*moltiplica;
				sotto=zeros(p,nonnulle);
				sotto(Ii,:)=eye(nonnulle);    
				Xtilde=cat(1, XH0, sotto);
			end
			Xfg=Xf(:,Ii);
			Beta=moltiplica*inv(Xtilde'*Xtilde)*Xtilde'*Ytilde;
			Yfpred=Yfpred+Xfg*Beta*PostGamD(i)/costante;
			I=union(I,Ii);
		end
		residual=Yf-Yfpred;

	case 'RBMA'
%--------------------------------> Bayes model averaging with 'restr' variables.
		I=restr;
		[k]=auto(X,pct);
		P=k*eye(p);  	 	 % matrix P=kI 
		restrc=setdiff([1:p],restr);
		expX=zeros(nOssP,p);
		expX(:,restr)=Xf(:,restr);
		expX(:,restrc)=Xf(:,restr)*inv(X(:,restr)'*X(:,restr)+P(restr,restr))* ...
			X(:,restr)'*X(:,restrc);
		Yfpred=expX*EMg;
		residual=Yf-Yfpred;
	end	
MSE=(diag(residual'*residual)./nOssP)';
