function [minpred,modelli]=ricerca(EMg,MargGam,cutoff,pct,X,thresh)

%ricerca:  Bayesian Variable Selection
%          Search selection loss for model averaging with restriction
%	   using either complete enumeration or backward elimination	
%***********************************************************************
%INPUTS:
% EMg,		approx expected value of the regression coeffs 
%		Bayes estimate (from atteso.m)
% MargGam,	Marginal probabilities (from marginale.m)
% cutoff,	cutoff point on marginals to choose the initial
%		model of the search	
% pct,		lower pct-tile of non-zero eigenvalues of X
%		is computed and used as hyperparameter
% X,		data
% thresh,	thresh=[] forces backward elimination 
%		to accept the least worse model
%
%OUTPUTS:
% modelli,      cell of searched models
% minpred,	selection loss values of searched models
%
%USAGE:
% [minpred,modelli]=ricerca(EMg,MargGam, cutoff,pct,X,Xf,thresh)
%
%NOTES:
% The loss is L=eye(q)
% The search is done by complete enumeration if the initial model 
%  has less than 15 vars or by Backward elimination otherwise.
% plot(ro90(rot90(minpred))) should produce an "elbow" plot.
% Functions called: auto, genera 
%
%
%Copyright (c) 1997-2003 Marina Vannucci.
%**********************************************************************


[nOss, p]=size(X);      
[q]=size(EMg,2);
L=eye(q);

m=find(MargGam>cutoff);  % select m by thresholding marginals 
mbar=setdiff([1:p],m);   % mbar is the complement of m

modelli{1}=m;
Kx=auto(X,pct);
P=Kx*eye(p);  	 	 
V=P+X'*X; 		 			
primo=V(mbar,mbar)-V(mbar,m)*inv(V(m,m))*V(m,mbar);			 
EMgmbar=EMg(mbar,:);			 
minpred=trace(L*EMgmbar'*primo*EMgmbar);    %--- formula (19)

if length(m)<=15         % if the number of variables in 'm' is less than 15
			 % then we do complete enumeration
	models=genera(length(m));
	modelli=cat(1, modelli, models);

	for j=2:length(modelli)			 
		m1=modelli{j};
		m1bar=setdiff([1:p],m1);	
		primo=V(m1bar,m1bar)-V(m1bar,m1)*inv(V(m1,m1))*V(m1,m1bar);
		EMgmbar=EMg(m1bar,:);
		minpred=[minpred trace(L*EMgmbar'*primo*EMgmbar)];
	end
	[minpred,minb]=sort(minpred);	
	modelli=modelli(minb);
	
else			 % otherwise backward elimination - 
			 % delete from 'm' a single
			 % variable at a time and compute (19). 
			 % Compare the lowest value
			 % with the value obtained for model 'm'.
	stop=0;
	while stop==0	
		valore=[]; 
		disp(' ')
		disp('Searching dimension:')
		disp(length(m)')
		disp('...')		
		for j=1:length(m)			 
			m1=m;
			m1(j)=[];
			m1bar=setdiff([1:p],m1);	
			primo=V(m1bar,m1bar)-V(m1bar,m1)*inv(V(m1,m1))*V(m1,m1bar);
			EMgmbar=EMg(m1bar,:);
			valore(j)=trace(L*EMgmbar'*primo*EMgmbar);
		end
		[mina,minb]=min(valore);

			% If 'thresh' is not [], 
			% then update 'm' if lowest is better (or slightly worse)

		if ~isempty(thresh)
			if mina<minpred+thresh 		
				minpred=[minpred mina];
				m(minb)=[];
				modelli{length(modelli)+1}= m;
			else
				stop=1;
			end	
		else
		
		% Otherwise, always accept the lowest

			minpred=[minpred mina];
			m(minb)=[];
			modelli{length(modelli)+1}= m;
		end	
		if length(m)==1 stop=1;
		else    
		end
	end	
end
