function [modelli,minpred]=sglpred(flag,X,Y,EMg,Hg,GammaD,MargGam,cutstep)

%sglpred: Bayesian Variable Selection
%         Search weighted loss for single model prediction 
%***************************************************************
%INPUTS:
% flag, 	- 1 to search through ordered visited models
%		- 2 to search through thresholded models
% X,Y,		training data
% EMg,		approx expected value of the regression coeffs 
%		Bayes estimate (from atteso.m)
% Hg,		hyperparameter of the prior on regression coefficients
%		- scalar (g-prior)
%		- vector of std's (diagonal prior)
%		- full matrix (non-diagonal prior)
% GammaD,	ordered visited models
% MargGam,	marginal probabilities
% cutstep,	threshold models are for .5:-cutstep:.001
%
%OUTPUTS:
% modelli,      cell of searched models
% minpred,	weigthed loss values of searched models
%
%USAGE:	
% [modelli,minpred]=sglpred(1,X,Y,EMg,Hg,GammaD,[],[])
% [modelli,minpred]=sglpred(2,X,Y,EMg,Hg,[],MargGam,cutstep)
%
%
%Copyright (c) 1997-2003 Marina Vannucci.
%**********************************************************************

[nOss, p]=size(X);       % p is number of variables 
[q]=size(Y,2);           % q is number of responses     
L=eye(q);

switch flag
case 1  % search through ordered visited models

	modelli=GammaD;
	switch class(GammaD)
		case 'cell'
			conta=length(GammaD);
		otherwise
			conta=size(GammaD,1);
	end
	for i=1:conta
		switch class(GammaD)
			case 'cell'
				m=str2num(GammaD{i});
			otherwise
				m=find(GammaD(i,:));
		end
		Xg=X(:,m);
		Bg=zeros(p,q);
		if (size(Hg,1)==1 & size(Hg,2)==1)
			%--------------------------- Hg scalar - gPrior 
			Hg2=Hg/(Hg+1);
			Bg(m,:)=Hg2*inv(Xg'*Xg)*Xg'*Y;
							
		elseif (size(Hg,1)==1 | size(Hg,2)==1)	
			%--------------------------- Hg vector of std's 
			
			Hg2=diag(Hg(m).^2);
			Bg(m,:)=inv(eye(length(m))+Hg2*Xg'*Xg)*Hg2*Xg'*Y;
		else
			%--------------------------- Hg full matrix
			Hg2=Hg(m,m);
			Bg(m,:)=inv(eye(length(m))+Hg2*Xg'*Xg)*Hg2*Xg'*Y;			  
		end
		minpred(i)=trace((EMg-Bg)'*X'*X*(EMg-Bg)*L);
	end
case 2  % search through thresholded models

	cutp=.5:-cutstep:.001;
	disp(' ')
	disp('Searching through')
	disp(length(cutp))
	disp('threshold models')
	disp('...')
	m=find(MargGam>=cutp(1));
	modelli{1}=m;
	Xg=X(:,m);
	Bg=zeros(p,q);
	if (size(Hg,1)==1 & size(Hg,2)==1)
		%--------------------------- Hg scalar - gPrior 
		Hg2=Hg/(Hg+1);
		Bg(m,:)=Hg2*inv(Xg'*Xg)*Xg'*Y;
						
	elseif (size(Hg,1)==1 | size(Hg,2)==1)	
		%--------------------------- Hg vector of std's 
		
		Hg2=diag(Hg(m).^2);
		Bg(m,:)=inv(eye(length(m))+Hg2*Xg'*Xg)*Hg2*Xg'*Y;
	else
			%--------------------------- Hg full matrix
		Hg2=Hg(m,m);
		Bg(m,:)=inv(eye(length(m))+Hg2*Xg'*Xg)*Hg2*Xg'*Y;			  
	end	
	minpred(1)=trace((EMg-Bg)'*X'*X*(EMg-Bg)*L);
	jj=1;
	for i=2:length(cutp)
		m=find(MargGam>=cutp(i));
		
		if isempty(setxor(m,modelli{jj}))
		else
			modelli{jj+1}=m;
			Xg=X(:,m);
			Bg=zeros(p,q);
			if (size(Hg,1)==1 & size(Hg,2)==1)
			%--------------------------- Hg scalar - gPrior 
				Hg2=Hg/(Hg+1);
				Bg(m,:)=Hg2*inv(Xg'*Xg)*Xg'*Y;
						
			elseif (size(Hg,1)==1 | size(Hg,2)==1)	
			%--------------------------- Hg vector of std's 
		
				Hg2=diag(Hg(m).^2);
				Bg(m,:)=inv(eye(length(m))+Hg2*Xg'*Xg)*Hg2*Xg'*Y;
			else
			%--------------------------- Hg full matrix
				Hg2=Hg(m,m);
				Bg(m,:)=inv(eye(length(m))+Hg2*Xg'*Xg)*Hg2*Xg'*Y;			  
			end		
			minpred=[minpred trace((EMg-Bg)'*X'*X*(EMg-Bg)*L)];
			jj=jj+1;
		end
	end	
end

